# Praat vowelchart plugin
# v1.2, 2017-11-15
# http://praatpfanne.lingphon.net/
#
# changelog
# v1.2: added option to fetch symbols from TextGrid; bug fixes


settingsFile$ = temporaryDirectory$ + "/vowelchart_settings.txt"
# read settings if available, else set defaults
if fileReadable (settingsFile$)
	settings$ = readFile$ (settingsFile$)
	speaker = extractNumber (settings$, "Speaker:")
	symbol$ = extractWord$ (settings$, "Symbol:")
	color = extractNumber (settings$, "Color:")
	draw_circle = extractNumber (settings$, "DrawCircle:")
	take_symbols_from = extractNumber (settings$, "TakeSymbolsFrom:")
	target_tier = extractNumber (settings$, "TargetTier:")
else
	speaker = 1
	symbol$ = "x"
	color = 2
	draw_circle = 1
	take_symbols_from = 1
	target_tier = 1
endif
activateTiers = 0

edInfo$ = Editor info
type$ = extractWord$ (edInfo$, "Editor type:")
if type$ = "TextGridEditor"
	gridID = extractNumber (edInfo$, "Editor name:")
	endeditor
	selectObject: gridID
	noTiers = Get number of tiers
	if noTiers > 0
		activateTiers = 1
	endif
	editor
endif


# input form
beginPause: "Vowelchart: Add vowel"
	choice: "Speaker", speaker
		option: "female"
		option: "male"
	if activateTiers = 1
		choice: "Take symbols from", take_symbols_from
			option: "input (type symbol below)"
			option: "TextGrid (select tier below)"
	endif
	word: "Symbol", symbol$
	if activateTiers = 1
		optionMenu: "Target tier", target_tier
		for i to noTiers
			option: string$ (i)
		endfor
	endif
	optionMenu: "Color", color
		option: "Black"
		option: "Red"
		option: "Green"
		option: "Blue"
		option: "Yellow"
		option: "Cyan"
		option: "Magenta"
		option: "Maroon"
		option: "Lime"
		option: "Navy"
		option: "Teal"
		option: "Purple"
		option: "Olive"
		option: "Pink"
		option: "Silver"
		option: "Grey"
	boolean: "Draw circle", draw_circle
clicked = endPause: "Cancel", "OK", 2, 1
if clicked = 1
	exitScript ()
endif

# write new settings
writeFileLine: settingsFile$, "Speaker: ", speaker
appendFileLine: settingsFile$, "Symbol: ", symbol$
appendFileLine: settingsFile$, "Color: ", color
appendFileLine: settingsFile$, "DrawCircle: ", draw_circle
appendFileLine: settingsFile$, "TakeSymbolsFrom: ", take_symbols_from
appendFileLine: settingsFile$, "TargetTier: ", target_tier

### show spectrogram and formants
# set maximum formant (Hz)
if speaker = 1
	formantMax = 5500
else
	formantMax = 5000
endif
# query editor info
specShow$ = extractWord$ (edInfo$, "Spectrogram show:")
formantShow$ = extractWord$ (edInfo$, "Formant show:")
if specShow$ = "no"
	Show spectrogram
endif
if formantShow$ = "no"
	formantPoles = 10
	formantWindow = 0.025
	formantRange = 30
	formantDot = 1
	formantPreemp = 50
	Show formants
else
	formantPoles = extractNumber (edInfo$, "Formant number of poles:")
	formantWindow = extractNumber (edInfo$, "Formant window length:")
	formantRange = extractNumber (edInfo$, "Formant dynamic range:")
	formantDot = extractNumber (edInfo$, "Formant dot size:")
	formantPreemp = extractNumber (edInfo$, "Formant pre-emphasis from:")
endif
# specify formant settings
Formant settings: formantMax, formantPoles / 2, formantWindow, formantRange, formantDot
Advanced formant settings: "Burg", formantPreemp

# query formants
x = Get first formant
f1 = round(x)
x = Get second formant
f2 = round(x)
# query cursor time
ctime = Get cursor

# switch to shell environment
endeditor

# if symbol should be taken from TextGrid get it
if activateTiers = 1 and take_symbols_from = 2
	isint = Is interval tier: target_tier
	if isint = 1
		interval = Get interval at time: target_tier, ctime
		symbol$ = Get label of interval: target_tier, interval
	else
		point = Get nearest index from time: target_tier, ctime
		symbol$ = Get label of point: target_tier, point
	endif
endif

# adjust picture settings
do: "Times"
Font size: 12
Select outer viewport: 0, 9, 0, 8
Axes: 3500, 0, 1100, 0
Line width: 1
do: "Solid line"
Colour: color$

# add vowel
Text special: f2, "centre", f1, "half", "Times", 12, "0", symbol$
if draw_circle = 1
	Draw circle: f2, f1, 50
endif


